package com.xydl.util;

import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;

@Component
@Slf4j
public class MqttUtil {

    @Value("${mqtt.broker}")
    private String broker;

    @Value("${mqtt.topic}")
    private String topic;

    @Value("${mqtt.username}")
    private String username;

    @Value("${mqtt.password}")
    private String password;

    @Value("${mqtt.publish.clientid}")
    private String publishClientid;

    @Value("${mqtt.subscribe.clientid}")
    private String subscribeClientid;

    @Value("${mqtt.qos}")
    private int qos;

    private MqttClient client;

    private MqttDeliveryToken token;

    public void connect() throws Exception {
        client = new MqttClient(broker, publishClientid, new MemoryPersistence());
        MqttConnectOptions options = new MqttConnectOptions();
        options.setUserName(username);
        options.setPassword(password.toCharArray());
        options.setConnectionTimeout(60);
        options.setKeepAliveInterval(30);
        client.connect(options);
    }

    public void disconnect() {
        try {
            client.disconnect();
            client.close();
        } catch (MqttException e) {
        }

    }

    public boolean publish(String content) throws Exception {
        MqttMessage message = new MqttMessage(content.getBytes());
        message.setQos(qos);
        token = client.getTopic(topic).publish(message);
        token.waitForCompletion();
        return token.isComplete();
    }


    public boolean publish2MQTT(String content) {

        MqttMessage message = new MqttMessage(content.getBytes());
        message.setQos(qos);
        // publish message
        try {
            token = client.getTopic(topic).publish(message);
            token.waitForCompletion();
        } catch (MqttException e) {
            throw new RuntimeException(e);
        }

        return token.isComplete();
    }

    public void subScribeMQTT() {

        try {
            MqttClient client = new MqttClient(broker, subscribeClientid, new MemoryPersistence());
            // connect options
            MqttConnectOptions options = new MqttConnectOptions();
            options.setUserName(username);
            options.setPassword(password.toCharArray());
            options.setConnectionTimeout(60);
            options.setKeepAliveInterval(30);
            // setup callback
            client.setCallback(new MqttCallback() {
                public void connectionLost(Throwable cause) {
                    log.info("connectionLost:{}", cause.getMessage());
                }

                public void messageArrived(String topic, MqttMessage message) {
                    log.info("topic:{} ", topic);
                    log.info("Qos:{} ", message.getQos());
                    log.info("{}", new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()) + " message content接收: " + new String(message.getPayload()));

                }

                public void deliveryComplete(IMqttDeliveryToken token) {
                    log.info("deliveryComplete---------{}", token.isComplete());
                }
            });
            client.connect(options);
            client.subscribe(topic, qos);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


}
